@@ -94,6 +94,9 @@ tests =
     , testCase "Issue #155" issue155
     , testCase "Issue #180" issue180
     , testCase "Issue #192" issue192
+    , testCase "Issue #230" issue230
+    , testCase "Issue #242" issue242
+    , testCase "Issue #257" issue257
     ] ++
     -- Some tests that won't work on Windows.
     if os /= "cygwin32" && os /= "mingw32"
@@ -114,6 +117,24 @@ call_capital = c2hsShelly $ chdir "tests/bugs/call_capital" $ do
   let expected = ["upper C();", "lower c();", "upper C();"]
   liftIO $ assertBool "" (T.lines res == expected)
 
+issue257 :: Assertion
+issue257 = c2hsShelly $ chdir "tests/bugs/issue-257" $ do
+  mapM_ rm_f ["Issue257.hs", "Issue257.chs.h", "Issue257.chs.c", "Issue257.chi",
+              "issue257_c.o", "Issue257.chs.o", "Issue257"]
+  cmd "c2hs" "Issue257.chs"
+  cmd cc "-c" "-o" "issue257_c.o" "issue257.c"
+  cmd cc "-c" "Issue257.chs.c"
+  cmd "ghc" "--make" "issue257_c.o" "Issue257.chs.o" "Issue257.hs"
+  res <- absPath "./Issue257" >>= cmd
+  let expected = ["True","False","True","False"]
+  liftIO $ assertBool "" (T.lines res == expected)
+
+issue242 :: Assertion
+issue242 = expect_issue 242 ["1"]
+
+issue230 :: Assertion
+issue230 = expect_issue 230 ["1", "2", "3", "4.0", "5", "6", "True", "8.0"]
+
 issue192 :: Assertion
 issue192 = hs_only_build_issue 192
 
@@ -455,17 +476,17 @@ do_issue_build strict cbuild n suff ext c2hsargs =
     run "c2hs" $ c2hsargs ++ [toTextIgnore $ uc <.> "chs"]
     code <- lastExitCode
     when (code == 0) $ do
-      when cbuild $ cmd cc "-c" "-o" (lcc <.> "o") (lc <.> "c")
+      when cbuild $ cmd cc "-c" "-o" (T.pack $ lcc <.> "o") (T.pack $ lc <.> "c")
       code <- lastExitCode
       when (code == 0) $ case (strict, cbuild) of
         (True, True) ->
-          cmd "ghc" "-Wall" "-Werror" "--make" (lcc <.> "o") (uc <.> "hs")
+          cmd "ghc" "-Wall" "-Werror" "--make" (T.pack $ lcc <.> "o") (T.pack $ uc <.> "hs")
         (False, True) ->
-          cmd "ghc" "--make" (lcc <.> "o") (uc <.> "hs")
+          cmd "ghc" "--make" (T.pack $ lcc <.> "o") (T.pack $ uc <.> "hs")
         (True, False) ->
-          cmd "ghc" "-Wall" "-Werror" "--make" (uc <.> "hs")
+          cmd "ghc" "-Wall" "-Werror" "--make" (T.pack $ uc <.> "hs")
         (False, False) ->
-          cmd "ghc" "--make" (uc <.> "hs")
+          cmd "ghc" "--make" (T.pack $ uc <.> "hs")
 
 expect_issue :: Int -> [Text] -> Assertion
 expect_issue n expected = expect_issue_with True True n "" [] expected
